"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _jsdoctypeparser = require("jsdoctypeparser");

var _iterateJsdoc = _interopRequireDefault(require("../iterateJsdoc"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const strictNativeTypes = ['undefined', 'null', 'boolean', 'number', 'string', 'Array', 'object', 'RegExp', 'Date', 'Function'];

var _default = (0, _iterateJsdoc.default)(({
  jsdocNode,
  sourceCode,
  report,
  utils,
  context
}) => {
  const jsdocTags = utils.filterTags(tag => {
    return utils.isTagWithType(tag.tag);
  });

  const preferredTypes = _lodash.default.get(context, 'settings.jsdoc.preferredTypes');

  const optionObj = context.options[0];

  const noDefaults = _lodash.default.get(optionObj, 'noDefaults');

  jsdocTags.forEach(jsdocTag => {
    const invalidTypes = [];
    let typeAst;

    try {
      typeAst = (0, _jsdoctypeparser.parse)(jsdocTag.type);
    } catch (error) {
      return;
    }

    (0, _jsdoctypeparser.traverse)(typeAst, node => {
      if (['NAME', 'ANY'].includes(node.type)) {
        const nodeName = node.type === 'ANY' ? '*' : node.name;
        let preferred;

        if (preferredTypes && _lodash.default.get(preferredTypes, nodeName) !== undefined) {
          const preferredSetting = preferredTypes[nodeName];

          if (!preferredSetting) {
            invalidTypes.push([nodeName]);
          } else if (typeof preferredSetting === 'string') {
            preferred = preferredSetting;
            invalidTypes.push([nodeName, preferred]);
          } else if (typeof preferredSetting === 'object') {
            preferred = _lodash.default.get(preferredSetting, 'replacement');
            invalidTypes.push([nodeName, preferred, _lodash.default.get(preferredSetting, 'message')]);
          }
        } else if (!noDefaults && node.type === 'NAME') {
          for (var _i = 0, _strictNativeTypes = strictNativeTypes; _i < _strictNativeTypes.length; _i++) {
            const strictNativeType = _strictNativeTypes[_i];

            if (strictNativeType.toLowerCase() === nodeName.toLowerCase() && strictNativeType !== nodeName && ( // Don't report if user has own map for a strict native type
            !preferredTypes || _lodash.default.get(preferredTypes, strictNativeType) === undefined)) {
              preferred = strictNativeType;
              invalidTypes.push([nodeName, preferred]);
              break;
            }
          }
        }

        if (preferred) {
          if (node.type === 'ANY') {
            node.type = 'NAME';
          }

          node.name = preferred;
        }
      }
    });

    if (invalidTypes) {
      const fixedType = (0, _jsdoctypeparser.publish)(typeAst);
      const tagName = jsdocTag.tag;
      invalidTypes.forEach(([badType, preferredType = '', message]) => {
        const fix = fixer => {
          return fixer.replaceText(jsdocNode, sourceCode.getText(jsdocNode).replace('{' + jsdocTag.type + '}', '{' + fixedType + '}'));
        };

        const tagValue = jsdocTag.name ? ' "' + jsdocTag.name + '"' : '';
        report(message || 'Invalid JSDoc @' + tagName + tagValue + ' type "' + badType + (preferredType ? '"; prefer: "' + preferredType : '') + '".', preferredType ? fix : null, jsdocTag, message ? {
          badType,
          preferredType,
          tagName,
          tagValue
        } : null);
      });
    }
  });
}, {
  meta: {
    fixable: 'code',
    type: 'suggestion'
  }
});

exports.default = _default;
module.exports = exports.default;
//# sourceMappingURL=checkTypes.js.map